/*
#
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
# or http://www.opensolaris.org/os/licensing.
# See the License for the specific language governing permissions
# and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each
# file and include the License file at usr/src/OPENSOLARIS.LICENSE.
# If applicable, add the following below this CDDL HEADER, with the
# fields enclosed by brackets "[]" replaced with your own identifying
# information: Portions Copyright [yyyy] [name of copyright owner]
#
# CDDL HEADER END
#
*/

/*
#
# Copyright (c) 2010. Oracle and/or its affiliates. All rights reserved.
# Use is subject to license terms.
#
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib/gi18n.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "about.h"
#include "../config.h"

void
show_about_dialog (GtkWidget *parent)
{
  const gchar *authors[] = {
    "Pu Chen <Pu.Chen@oracle.com>",
    "Shinobu Matsuzuka <Shinobu.Matsuzuka@oracle.com>",
    NULL
  };

  const gchar *documenters[] = {
    "Pu Chen <Pu.Chen@oracle.com>",
    "Shinobu Matsuzuka <Shinobu.Matsuzuka@oracle.com>",
    NULL
  };

  const gchar *translator_credits = _("translator-credits");
  const gchar *copyright = "Copyright (c) 2011, Oracle and/or its affiliates. All right reserved";
  const gchar *comments = _("Solaris Command Assistant");
  
  const gchar *license =
    "This program is released under: \n"
    "CDDL version 1.0 & gSOAP Public License \n";

  g_return_if_fail (GTK_IS_WIDGET (parent));

  gchar* logo_name = g_strdup_printf ("%s/command_assistant_applet.png", CMDASSIST_PIXDIR);
  GdkPixbuf* logo = gdk_pixbuf_new_from_file(logo_name, NULL);
  g_free(logo_name);
  
  gtk_show_about_dialog (GTK_IS_WINDOW (parent) ? GTK_WINDOW (parent) : NULL,
  			 "name", _("Command Assistant"),
  			 "version", "2.1",
  			 "copyright", copyright,
  			 "comments", comments,
			 "logo", logo,
  			 "authors", authors,
  			 "documenters", documenters,
  			 "translator-credits", translator_credits,
  			 "license", license,
  			 "wrap-license", TRUE,
			 "screen", gtk_widget_get_screen (parent),
  			 NULL);
  
}
