/*
#
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
# or http://www.opensolaris.org/os/licensing.
# See the License for the specific language governing permissions
# and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each
# file and include the License file at usr/src/OPENSOLARIS.LICENSE.
# If applicable, add the following below this CDDL HEADER, with the
# fields enclosed by brackets "[]" replaced with your own identifying
# information: Portions Copyright [yyyy] [name of copyright owner]
#
# CDDL HEADER END
#
*/

/*
#
# Copyright (c) 2010. Oracle and/or its affiliates. All rights reserved.
# Use is subject to license terms.
#
*/

#include <locale.h>
#include "soapH.h"
#include "service.h"
#include "../common.h"

#include "EnginePortBinding.nsmap"

static void
get_hostname(const gchar *src, gchar *n){
	gchar tmp[100];
	strncpy(tmp, &src[7], strlen(src)-7); 
	gint p = strstr(tmp, "/") - tmp;
	strncpy(n, tmp, p);
	n[p] = '\0';
}

gboolean test_service(GList *doc_list, cmdassist_config *conf) {
	gchar hostname[50];
	gboolean rs = FALSE;
        struct soap soap;

        soap_init(&soap);
        soap_set_mode(&soap, SOAP_C_UTFSTRING);

	get_hostname(conf->system.wsdl, hostname);

	soap.fopen(&soap, conf->system.wsdl, hostname, 80);
	if (soap.error) {
		//soap_print_fault(&soap, stderr);
	} else {
		rs = TRUE;
	}

        soap_destroy(&soap);
        soap_end(&soap);
        soap_done(&soap);

        return rs;
}

GList* get_doc_list(GList *doc_list, cmdassist_config *conf) {
	struct soap soap;
	soap_init(&soap);
	soap_set_mode(&soap, SOAP_C_UTFSTRING);

	struct ns1__getDocumentsResponse ret;
	struct ns1__getDocuments search;

	debug_printf("%s\n", "Start to connect service");
	if (soap_call___ns1__getDocuments(&soap, conf->system.wsdl, NULL, &search, &ret) == SOAP_OK) {
		debug_printf("%s\n", "connect server is OK");
	 	int i = 0; 
		while(ret.__sizereturn_ > i) {
			cmd_doc_list *doc = g_malloc(sizeof(cmd_doc_list));
			strcpy(doc->doc_name, ret.return_[i].name);
			strcpy(doc->doc_des, ret.return_[i].des);

			doc_list = g_list_append(doc_list, doc);
			i++;
		}

	}
	else {
		//soap_print_fault(&soap, stderr);
		soap_destroy(&soap);
		soap_end(&soap);
		soap_done(&soap);
		return NULL;
	}
	
	soap_destroy(&soap);
	soap_end(&soap);
	soap_done(&soap);

	return doc_list;
}

int getResult(char *str, cmdassist_config *conf, SearchResult *command)
{

	static struct soap soap;

	struct ns1__doSearchResponse ret;

	soap_init(&soap);
	debug_printf("0 %s\n", "Start to connect service");
	soap_set_mode(&soap, SOAP_C_UTFSTRING);
	debug_printf("1 %s\n", "Start to connect service");
	
	char *locale = setlocale(LC_ALL, "");	
	struct ns1__doSearch srt;
	struct ns1__SearchRequestType search;
	search.Str = str;
	if(conf->system.languageID == 0) {
		search.Locale = locale;
	} else {
		search.Locale = conf->system.language;
	}
	search.IsCommand = conf->criteria.command;
	search.IsTitle = conf->criteria.title;
	search.IsContent = conf->criteria.content;
	search.MaxSize = conf->criteria.result_size;
	search.DocList = conf->doc.docs;
	srt.request = &search;

	debug_printf("%s: %s\n", "Start to connect service", conf->system.wsdl);
	if (soap_call___ns1__doSearch(&soap, conf->system.wsdl, NULL, &srt, &ret) == SOAP_OK) {
		command->size = ret.__sizereturn_;
		debug_printf("%d\n", "result:", command->size);
	        if(command->list != NULL) {
                	free(command->list);
                	command->list = NULL;
        	}
		if(command->size != 0) {
			command->list = malloc(ret.__sizereturn_*sizeof(ResultItem));
		}

		int i = 0; 
                while(ret.__sizereturn_ > i) {

                                memset(command->list[i].type, 0, sizeof(command->list[i].type));
                                strncpy(command->list[i].type, ret.return_[i].DocType, sizeof(command->list[i].type) -1);

                                memset(command->list[i].url, 0, sizeof(command->list[i].url));
                                strncpy(command->list[i].url, ret.return_[i].Url, sizeof(command->list[i].url)-1);

                                memset(command->list[i].command, 0, sizeof(command->list[i].command));
                                strncpy(command->list[i].command, ret.return_[i].Command, sizeof(command->list[i].command)-1);

                                memset(command->list[i].title, 0, sizeof(command->list[i].title));
                                strncpy(command->list[i].title, ret.return_[i].Title, sizeof(command->list[i].title)-1);

                                memset(command->list[i].score, 0, sizeof(command->list[i].score));
                                strncpy(command->list[i].score, ret.return_[i].Title, sizeof(command->list[i].score)-1);

                                memset(command->list[i].desc, 0, sizeof(command->list[i].desc));
                                strncpy(command->list[i].desc, ret.return_[i].Content, sizeof(command->list[i].desc)-1);

                                memset(command->list[i].size, 0, sizeof(command->list[i].size));
                                strncpy(command->list[i].size, ret.return_[i].Title, sizeof(command->list[i].size)-1);

			i++;
                }

	}
	else {
		//soap_print_fault(&soap, stderr);
		soap_destroy(&soap);
		soap_end(&soap);
		soap_done(&soap);
		return 1;
	}
	debug_printf("%s %d\n", "result size:", command->size);
	soap_destroy(&soap);

	soap_end(&soap);

	soap_done(&soap);

	return 0;
}
