/*
#
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
# or http://www.opensolaris.org/os/licensing.
# See the License for the specific language governing permissions
# and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each
# file and include the License file at usr/src/OPENSOLARIS.LICENSE.
# If applicable, add the following below this CDDL HEADER, with the
# fields enclosed by brackets "[]" replaced with your own identifying
# information: Portions Copyright [yyyy] [name of copyright owner]
#
# CDDL HEADER END
#
*/

/*
#
# Copyright (c) 2010. Oracle and/or its affiliates. All rights reserved.
# Use is subject to license terms.
#
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "common.h"
#include <regex.h>
#include "tree-search.h"

static Treenode *treeRoot = NULL;

static void
treeInsert(Treenode *root, char *word, char *title, char *url, char *type)
{
    Treenode *p = root;
    int i=0;
    char ch;

    while(word[i])
    {
        ch = (char)word[i];
	char *re = strchr(p->child, ch);
        if(re == NULL) {
		int count = strlen(p->child);
		p->child[count] = ch;
		p->child[count+1] = '\0';
		p->count++ ;

		Treenode *node = (Treenode *)malloc(sizeof(Treenode));
		node->next = NULL;
		node->isLeaf = FALSE;
		node->leaf = NULL;
		node->count = 0;

		strcpy(node->child, "\0");
		Treenode **tmp = malloc((count + 1)*sizeof(Treenode *));
		for(int i=0; i<count; i++) {
			tmp[i] = (Treenode *)p->next[i];
		}
		tmp[count] = node;
		free(p->next);
		p->next = (Treenode **)tmp;
		p = (Treenode *)p->next[count];
	} else {
		p->count++;
		int pr = strlen(p->child) - strlen(re);
		p = (Treenode *)p->next[pr];
	}	
	i++;
    }
    p->isLeaf = TRUE;
    Leafnode *leaf = malloc(sizeof(Leafnode));
    strncpy(leaf->command, word, sizeof(leaf->command)-1);
    leaf->command[sizeof(leaf->command)-1] = '\0';
    strncpy(leaf->title, title, sizeof(leaf->title)-1);
    leaf->title[sizeof(leaf->title)-1] = '\0';

    strncpy(leaf->url, url, sizeof(leaf->url)-1);
    leaf->url[sizeof(leaf->url)-1] = '\0';
    strncpy(leaf->type, type, sizeof(leaf->type)-1); 
    leaf->type[sizeof(leaf->type)-1] = '\0';
    p->leaf = (Leafnode *)leaf;
}

static int
treeList(Treenode *p, ResultSet *rs, int max_size) {
	if(p == NULL) {
		return 0;
	}
	if(p->isLeaf) {
		if(rs->size >= max_size) {
			return 0;
		}
		strcpy(rs->rs[rs->size].command, p->leaf->command);
		strcpy(rs->rs[rs->size].title, p->leaf->title);
		strcpy(rs->rs[rs->size].url, p->leaf->url);
		strcpy(rs->rs[rs->size].type, p->leaf->type);
		rs->size++; 
		printf("%d : %s %s\n", rs->size, p->leaf->command, p->leaf->title);
	}
	int size = strlen(p->child);
	if(size == 0) {
		return 0;
	} else {
		for(int i=0; i<size; i++) {
			Treenode *q = (Treenode *)p->next[i];
			treeList(q, rs, max_size);
		}
	}
}

static int
treeDashSearch(Treenode *root, char *word, ResultSet *rs, int max_size) {
        Treenode *p = root;
        int i= 0, ch = 0;

        if(p == NULL)
                return 1;

	char command[20];
	char cond[20];
	char *pri = strchr(word, ' ');
	if(pri != NULL) {
		strncpy(command, word, pri - word);
		command[pri - word] = '\0'; 
		pri = strchr(pri, '-');
		if(!pri) 
			return 1;
		strncpy(cond, pri+1, sizeof(cond)-1);
		cond[sizeof(cond)-1] = '\0';
	}

        while(command[i])
        {
                ch = command[i];
                char *re = strchr(p->child, ch);
                if(re == NULL) {
                        return 0;
                }
                int pr = strlen(p->child) - strlen(re);
                p = (Treenode *)p->next[pr];
                i++;
        }
	printf("\n\nsize: %d  cond: %s\n\n", p->count, cond);
	ResultSet set;
	set.size = 0;
	set.rs = malloc(p->count * sizeof(Leafnode));
        treeList(p, &set, p->count);

	for(int i=0; i< set.size; i++) {
		char *sCommand = set.rs[i].command;
		if(strstr(sCommand, cond)) {
			if(rs->size >= max_size) {
				break;
			}
			printf("found: %s", sCommand);
			strcpy(rs->rs[rs->size].command, set.rs[i].command);
			strcpy(rs->rs[rs->size].title, set.rs[i].title);
			strcpy(rs->rs[rs->size].url, set.rs[i].url);
			strcpy(rs->rs[rs->size].type, set.rs[i].type);
			rs->size++; 
		}
		
	}

	free(set.rs);

	return 0;
}

int treeSearch(Treenode *root, char *word, ResultSet *rs, int max_size)
{
	Treenode *p = root;
	int i= 0, ch = 0;

	if(p == NULL) 
		return 1;

	while(word[i])
	{
		ch = word[i];
		char *re = strchr(p->child, ch);
		if(re == NULL) {
			return 0;
		}
		int pr = strlen(p->child) - strlen(re);
		p = (Treenode *)p->next[pr];
		i++;
	}
	treeList(p, rs, max_size);

	return 0;
}

static Treenode* 
initTree() {

	FILE *f;
	char buf[1000];
	char type[10];
	char url[80];
	char command[80];
	char title[100];

	gchar* datafile = g_strdup_printf ("%s/CommandAssistantIndex", CMDASSIST_DATADIR);
        f = fopen(datafile, "r");
        g_free (datafile);

	if( f == NULL) {
		printf("open file failure\n");
		return 0;
	} 
	Treenode *root = NULL;
	root = malloc(sizeof(Treenode));
	root->next = NULL;
	root->count = 0;
	root->isLeaf = FALSE;
	root->leaf = NULL;
	strcpy(root->child, "\0");
	fgets(buf, sizeof(buf), f);
	while (!feof(f)) {
                buf[strlen(buf)-1] = ' ';
                const char *split = "\t";
                char *p;
                p = strtok(buf, split);
                if(p != NULL){
                        strncpy(type, p, sizeof(type)-1);
                        type[sizeof(type)-1] = '\0';
                        p = strtok(NULL, split);
                }
                if(p !=NULL) {
                        strncpy(url, p, sizeof(url)-1);
                        url[sizeof(url)-1] = '\0';
                        p = strtok(NULL, split);
                }
                if(p != NULL){
                        strncpy(command, p, sizeof(command)-1);
                        command[sizeof(command)-1] = '\0';
                        p = strtok(NULL, split);
                }
                if(p != NULL){
                        strncpy(title, p, sizeof(title)-1);
                        title[sizeof(title)-1] = '\0';
                        p = strtok(NULL, split);
                }
                treeInsert(root, command, title, url, type);
                fgets(buf, sizeof(buf), f);

	}
	return root;
}

int doLocalSearch(char *str, cmdassist_config *conf, SearchResult *command, int max_size)
{
	if(treeRoot == NULL) {
		treeRoot = initTree();
	} 
        ResultSet rs;
       	rs.size = 0;
	rs.rs = malloc(max_size * sizeof(Leafnode));

	treeSearch(treeRoot, str, &rs, max_size);
	int size = rs.size;

	if(size == 0) {
		treeDashSearch(treeRoot, str, &rs, max_size);
	}

	size = rs.size ;
	command->size = size;
	command->list = malloc(size * sizeof(ResultItem));
    	for(int i=0; i<size; i++) {
		strncpy(command->list[i].type, rs.rs[i].type, sizeof(command->list[i].type) -1);
		command->list[i].type[sizeof(command->list[i].type) -1] = '\0';
		
		strncpy(command->list[i].url,  rs.rs[i].url, sizeof(command->list[i].url)-1);
		command->list[i].url[sizeof(command->list[i].url)-1] = '\0';

		strncpy(command->list[i].command,  rs.rs[i].command, sizeof(command->list[i].command)-1);
		command->list[i].command[sizeof(command->list[i].command)-1] = '\0';

		strncpy(command->list[i].title,  rs.rs[i].title, sizeof(command->list[i].title)-1);
		command->list[i].title[sizeof(command->list[i].title)-1] = '\0';

		strncpy(command->list[i].score, "0", sizeof(command->list[i].score)-1);

		strncpy(command->list[i].desc, "n/a", sizeof(command->list[i].desc)-1);

		strncpy(command->list[i].size, "0", sizeof(command->list[i].size)-1);
    	}
	if(rs.rs != NULL) {
		free(rs.rs);
		rs.rs = NULL;
	}
	
	return 0;
}
